import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "CB.GroupManager.Strings";
--	self.LabelCaption:SetText(Resource[language][5]..":");

local LABEL_WIDTH = 150;
local ROW_HEIGHT  = 20;
local ROW_SPACING = 3;
local BORDER_MARGIN = 5;
local MARGIN_TOP = 15;
local MARGIN_BOTTOM = 0;

local XoffsetCol1 = BORDER_MARGIN;
local XoffsetCol2 = BORDER_MARGIN + LABEL_WIDTH + ROW_SPACING;
local XoffsetCol3 = BORDER_MARGIN + LABEL_WIDTH + ROW_SPACING + ROW_HEIGHT + ROW_SPACING;

local players = {};
local labels = {};
local auto_fetch = false;


language=1;
locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end

function AddCallback(object, event, callback)
	if (object[event] == nil) then
		object[event] = callback;
	else
		if (type(object[event]) == "table") then
			table.insert(object[event], callback);
		else
			object[event] = {object[event], callback};
		end
	end
end

function analyzeMessage(text, message)
	local idx = string.find(message, text, 1, true);
	if ( idx ~= nil and idx > 1) then
		return true;
	else
		return false;
	end
end

function addButton(win, x, y, width, height, r, g, b, alias)
	local qs = Turbine.UI.Lotro.Quickslot();
	qs:SetParent(win);
	qs:SetSize(width,height);
	qs:SetPosition(x,y);
	qs:SetZOrder(1);
	qs:SetUseOnRightClick(false);
	qs.DragDrop = 	function(sender, args)
						resetWindow();
					end

	local btn = Turbine.UI.Control();
	btn:SetParent(gmWindow);
	btn:SetSize(width,height);
	btn:SetPosition(x,y);
	btn:SetZOrder(2);
	btn:SetBackColor(Turbine.UI.Color(r/255,g/255,b/255));
	btn:SetMouseVisible(false);

	local act = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias,nil);
	act:SetData(alias);
	qs:SetShortcut(act);
end

function addLabel(win, x, y, width, height, text)
	local lbl = Turbine.UI.Label();
	lbl:SetParent(win);
	lbl:SetSize(width, height);
	lbl:SetPosition(x, y);
	lbl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	lbl:SetText(text);
	lbl:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	return lbl;
end

function addIcon(win, x, y, width, height, iconPath)
	local ico = Turbine.UI.Control();
	ico:SetParent(win);
	ico:SetSize(width, height);
	ico:SetPosition(x, y);
	if not pcall(function() ico:SetBackground(iconPath); end) then ico:SetBackColor(Turbine.UI.Color(1,0,0)); end
	return ico;
end

function addRaidRow(win, offset)
	addLabel(win, XoffsetCol1 + ROW_HEIGHT + ROW_SPACING, offset+3, LABEL_WIDTH, ROW_HEIGHT, (Resource[language][2]));
	addButton(win, XoffsetCol2+1, offset+1, ROW_HEIGHT-2, ROW_HEIGHT-2, 0, 128, 0, (Resource[language][3]));
	addButton(win, XoffsetCol3+1, offset+1, ROW_HEIGHT-2, ROW_HEIGHT-2, 255, 128, 0, (Resource[language][4]));
end

function addReadyRow(win, offset)
	addLabel(win, XoffsetCol1 + ROW_HEIGHT + ROW_SPACING, offset+3, LABEL_WIDTH, ROW_HEIGHT, (Resource[language][5]));
	addButton(win, XoffsetCol2+1, offset+1, 2*ROW_HEIGHT-4+ROW_SPACING, ROW_HEIGHT-2, 0, 0, 255, (Resource[language][6]));
	--self.LabelCaption:SetText(Resource[language][5]);
end

function addRow(win, offset, player)
	addButton(win, XoffsetCol2+1, offset+1, ROW_HEIGHT-2, ROW_HEIGHT-2, 0, 255, 0, (Resource[language][7] .. player.Name));
	addButton(win, XoffsetCol3+1, offset+1, ROW_HEIGHT-2, ROW_HEIGHT-2, 255, 0, 0, (Resource[language][8] .. player.Name));
	addIcon(win, XoffsetCol1, offset, ROW_HEIGHT, ROW_HEIGHT, "CB/GroupManager/Resources/" .. player.Class .. ".tga");
	return addLabel(win, XoffsetCol1 + ROW_HEIGHT + ROW_SPACING, offset, LABEL_WIDTH, ROW_HEIGHT, player.Name);
end

function grabPlayers()
	local oldIdx = nil;
	if ( players ~= nil ) then
		oldIdx = {};
		for idx = 1,table.getn(players) do
			oldIdx[players[idx].Name] = idx;
		end
	end
	
	players = {};
	party = Turbine.Gameplay.LocalPlayer.GetInstance().GetParty();
	local selfIdx = nil;
	if party ~= nil then
		for idx = 1,party:GetMemberCount() do
			local obj = Turbine.Object();
			obj.Name = party:GetMember(idx):GetName();
			obj.Class = party:GetMember(idx):GetClass();
			table.insert(players, obj);
			if (Turbine.Gameplay.LocalPlayer:GetInstance():GetName() == obj.Name) then selfIdx = idx; end
		end
		players[1],players[selfIdx] = players[selfIdx],players[1];
		local sortPlayers = {}
		local newPlayers = {}
		for idx = 1, table.getn(players) do
			local p = oldIdx[players[idx].Name];
			if ( p ~= nil ) then
				sortPlayers[p] = players[idx];
			else
				table.insert(newPlayers, players[idx]);
			end
		end
		if ( table.getn(newPlayers) > 0 ) then
			local ins = 1;
			for idx = 1, table.getn(players) do
				if ( sortPlayers[idx] == nil ) then
					sortPlayers[idx] = newPlayers[ins];
					ins = ins + 1;
				end
			end
		end
		players = {};
		local newIdx = 1;
		for idx = 1,math.max(table.getn(sortPlayers), table.getn(oldIdx)) do
			if ( sortPlayers[idx] ~= nil ) then
				players[newIdx] = sortPlayers[idx];
				newIdx = newIdx + 1;
			end
		end
	else
		local obj = Turbine.Object();
		obj.Name = Turbine.Gameplay.LocalPlayer:GetInstance():GetName();
		obj.Class = Turbine.Gameplay.LocalPlayer:GetInstance():GetClass();
		players[1] = obj;
	end
	resetWindow();
end

function refreshLabels()
	for idx = 1,table.getn(players) do
		local playerName = players[idx].Name;
		if labels[playerName] ~= nil then labels[playerName]:SetForeColor(Turbine.UI.Color(255/255,0/255,0/255)); end
	end
	local party = Turbine.Gameplay.LocalPlayer:GetInstance():GetParty();
	if party ~= nil then
		for idx = 1,party:GetMemberCount() do
			if labels[party:GetMember(idx):GetName()] ~= nil then labels[party:GetMember(idx):GetName()]:SetForeColor(Turbine.UI.Color(0/255,255/255,0/255)); end
		end
	end
end

function registerPartyListeners()
	local myParty = Turbine.Gameplay.LocalPlayer:GetInstance():GetParty();
	if myParty ~= nil then
		AddCallback(myParty, (Resource[language][9]), refreshLabels);
		AddCallback(myParty, (Resource[language][10]), refreshLabels);
		
		-- Auto-fetch check
		if ( auto_fetch and myParty:GetMemberCount() == table.getn(players) ) then grabPlayers(); end
	end
	refreshLabels();

	AddCallback(Turbine.Chat,(Resource[language][11]),function(sender, args)
		if ( args.ChatType ~= Turbine.ChatType.Standard ) then return; end
		if ( analyzeMessage((Resource[language][12]), args.Message) or analyzeMessage((Resource[language][13]), args.Message) ) then reloadPlugin(); end
		
	end);	
end

selectedPlayerLabel = nil;
function doPlayerSwap(sender, args)
	if (selectedPlayerLabel == nil) then
		selectedPlayerLabel = sender;
		selectedPlayerLabel:SetText("[" .. players[selectedPlayerLabel.playerIndex].Name .. "]");
		selectedPlayerLabel:SetForeColor(Turbine.UI.Color(27/255,205/255,224/255));
	else
		if (sender.playerIndex ~= selectedPlayerLabel.playerIndex) then
			local tmpPlayer = players[selectedPlayerLabel.playerIndex];
			players[selectedPlayerLabel.playerIndex] = players[sender.playerIndex];
			players[sender.playerIndex] = tmpPlayer;
		end
		selectedPlayerLabel = nil;
		resetWindow();
	end
end

function resetWindow()
	if (players == nil or table.getn(players) == 0) then grabPlayers(); return; end
	gmWindow:GetControls():Clear();
	labels = {};
	selectedPlayerLabel = nil;
	local gmWidth = LABEL_WIDTH + 2 * ( BORDER_MARGIN + ROW_HEIGHT + ROW_SPACING );
	local gmHeight = 2 * BORDER_MARGIN + MARGIN_TOP + MARGIN_BOTTOM + ( ROW_HEIGHT + ROW_SPACING ) * ( math.max(1, table.getn(players)) + 3 ); 
	if isCollapsed then gmHeight = 10; end

	local yPos = MARGIN_TOP;
	for idx = 1,table.getn(players) do
		local pl = players[idx];
		if pl.Name ~= Turbine.Gameplay.LocalPlayer:GetInstance():GetName() then
			local lbl = addRow(gmWindow, yPos, pl);
			labels[pl.Name] = lbl;
			lbl.MouseClick = doPlayerSwap;
			lbl.playerIndex = idx;
			yPos = yPos + ROW_HEIGHT + ROW_SPACING;
		end
		if idx == 6 then
			addRaidRow(gmWindow, yPos);
			yPos = yPos + ROW_HEIGHT + ROW_SPACING;
		end
	end
	if table.getn(players) < 6 then
		addRaidRow(gmWindow, yPos);
		yPos = yPos + ROW_HEIGHT + ROW_SPACING;
	end
	addReadyRow(gmWindow, yPos);
	yPos = yPos + ROW_HEIGHT + ROW_SPACING;
	refreshLabels();

	local btn = Turbine.UI.Lotro.Button();
	btn:SetParent(gmWindow);
	btn:SetSize(LABEL_WIDTH, ROW_HEIGHT);
	btn:SetPosition((gmWidth - LABEL_WIDTH) / 2, yPos + ROW_SPACING);
	btn:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	btn:SetText((Resource[language][14]));
	btn.Click = grabPlayers;
				
	local chk = Turbine.UI.Lotro.CheckBox();
	chk:SetParent(gmWindow);
	chk:SetSize(LABEL_WIDTH, ROW_HEIGHT);
	chk:SetPosition((gmWidth - LABEL_WIDTH) / 2, yPos + 2*ROW_SPACING + ROW_HEIGHT)
	chk:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	chk:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	btn:SetText((Resource[language][15]));
	chk:SetChecked(auto_fetch);
	chk.CheckedChanged = function(sender, args)
		auto_fetch = sender:IsChecked();
	end
				
	local topBar = Turbine.UI.Label();
	topBar:SetParent(gmWindow);
	topBar:SetSize(gmWidth, 10);
	topBar:SetPosition(0,0);
	topBar:SetText((Resource[language][16]));
	topBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	topBar:SetForeColor(Turbine.UI.Color(0,0,0));
	topBar:SetBackColor(Turbine.UI.Color(0.8,0.8,0.8));
	topBar:SetMouseVisible(true);
	topBar.MouseDoubleClick = toggleCollapseWindow;
	topBar.MouseDown = startDrag;
	topBar.MouseUp = endDrag;
	topBar.MouseMove = doDrag;
	
	gmWindow:SetSize(gmWidth,gmHeight);
	gmWindow:SetText((Resource[language][17]));
	gmWindow:SetVisible(true);	
end

isCollapsed = false;
function toggleCollapseWindow()
	isCollapsed = not isCollapsed;
	resetWindow();
end

function startDrag(sender, args)
	startX = args.X;
	startY = args.Y;
	isDrag = true;
end

function endDrag(sender, args)
	if isDrag then
		gmWindow:SetLeft(gmWindow:GetLeft() + (args.X - startX));
		gmWindow:SetTop(gmWindow:GetTop() + (args.Y - startY));
		isDrag = false;
	end
end

function doDrag(sender, args)
	if isDrag then
		gmWindow:SetLeft(gmWindow:GetLeft() + (args.X - startX));
		gmWindow:SetTop(gmWindow:GetTop() + (args.Y - startY));
	end
end

function initWindowDrag(sender, args)
	isDrag = false;
	gmWindow.MouseUp = endDrag;
	gmWindow.MouseMove = doDrag;
end

function reloadPlugin()
	Turbine.PluginManager.LoadPlugin("GroupManagerReloader");
end

-- Create window
gmWindow = Turbine.UI.Window();
gmWindow:SetPosition(30,30);
gmWindow:SetBackColor(Turbine.UI.Color(0,0,0));
gmWindow:SetOpacity(0.7);
local persist = Turbine.PluginData.Load(Turbine.DataScope.Character, "Cee_GroupManager", nil);
if persist ~= nil then
	gmWindow:SetTop(persist.posTop);
	gmWindow:SetLeft(persist.posLeft);
	isCollapsed = persist.isCollapsed;
	players = persist.players;
	if ( persist.autoFetch ~= nil ) then auto_fetch = persist.autoFetch; end
	if ( players == nil or players["len"] ~= nil ) then grabPlayers(); else resetWindow(); end
else
	grabPlayers();
end
initWindowDrag();

-- Register PartyChanged event
AddCallback(Turbine.Gameplay.LocalPlayer.GetInstance(), (Resource[language][24]), registerPartyListeners);
registerPartyListeners();

-- Save state on unload
Plugins["GroupManager"].Unload = function(self,sender,args)
	local obj = Turbine.Object();
	obj.posTop = gmWindow:GetTop();
	obj.posLeft = gmWindow:GetLeft();
	obj.players = players;
	obj.isCollapsed = isCollapsed;
	obj.autoFetch = auto_fetch;
	Turbine.PluginData.Save(Turbine.DataScope.Character, "Cee_GroupManager", obj, nil);
end

gmWindow.Update = function()
	Turbine.PluginManager.UnloadScriptState("GroupManagerReloader");
	gmWindow:SetWantsUpdates(false);
end
gmWindow:SetWantsUpdates(true);

gmCmd = Turbine.ShellCommand();
gmCmd.GetShortHelp = function() return (Resource[language][18]); end
function gmCmd:Execute(cmd, args)
	if string.len(args) > 0 then
		args = string.lower(args);
		if ( string.find(args,(Resource[language][19])) == 1 ) then gmWindow:SetVisible(true); return; end
		if ( string.find(args,(Resource[language][20])) == 1 ) then gmWindow:SetVisible(false); return; end
	end
	Turbine.Shell.WriteLine((Resource[language][21]));
end
Turbine.Shell.AddCommand("gm", gmCmd);
Turbine.Shell.AddCommand("groupmanager", gmCmd);
